<?php

namespace Drupal\style_entity\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class StylesForm.
 */
class StylesForm extends EntityForm {

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $styles = $this->entity;

    $classes_count = $form_state->get('classes_count') ? $form_state->get('classes_count') : count($styles->get('classes'));
    if ($classes_count == 0) {
      $form_state->set('classes_count', 1);
      $classes_count = 1;
    }

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#maxlength' => 255,
      '#default_value' => $styles->label(),
      '#description' => $this->t("Label for the Styles."),
      '#required' => TRUE,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $styles->id(),
      '#machine_name' => [
        'exists' => '\Drupal\style_entity\Entity\Styles::load',
      ],
      '#disabled' => !$styles->isNew(),
    ];

    $form['classes'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Classes'),
      '#description' => $this->t('Add one more more classes to add to the container when this style is applied. To remove classes, delete the contents of the field and save.'),
      '#tree' => FALSE,
      '#attributes' => [
        'id' => 'classes',
      ],
    ];
    $form['classes']['classes'] = [
      '#tree' => TRUE,
    ];

    $classes = $styles->get('classes');
    for ($i = 0; $i < $classes_count; $i++) {
      $form['classes']['classes'][$i] = [
        '#type' => 'textfield',
        '#default_value' => !empty($classes) ? $classes[$i] : NULL,
      ];
    }

    $form['classes']['actions'] = [
      '#type' => 'actions',
    ];
    $form['classes']['actions']['add_class'] = [
      '#type' => 'submit',
      '#value' => $this->t('Add another'),
      '#submit' => ['::addClass'],
      '#ajax' => [
        'callback' => '::addCallback',
        'wrapper' => 'classes',
      ],
    ];

    $options = $styles->getUses();

    $form['uses'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Uses'),
      '#options' => $options,
      '#default_value' => $styles->get('uses'),
    ];

    return $form;
  }

  /**
   * Callback for both ajax-enabled buttons.
   */
  public static function addCallback(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $element = $trigger['#ajax']['wrapper'];
    return $form[$element];
  }

  /**
   * Submit handler for the "add-one-more" button.
   *
   * Increments the max counter and causes a rebuild.
   */
  public static function addClass(array &$form, FormStateInterface $form_state) {
    $classes = $form_state->getValue('classes');
    $classes_count = count($classes);
    $add_class_button = $classes_count + 1;
    $form_state->set('classes_count', $add_class_button);
    $form_state->setRebuild();
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $styles = $this->entity;
    $classes = $styles->get('classes');
    foreach ($classes as $key => $class) {
      if (empty($class)) {
        unset($classes[$key]);
      }
    }
    $styles->set('classes', $classes);
    $status = $styles->save();

    switch ($status) {
      case SAVED_NEW:
        drupal_set_message($this->t('Created the %label Styles.', [
          '%label' => $styles->label(),
        ]));
        break;

      default:
        drupal_set_message($this->t('Saved the %label Styles.', [
          '%label' => $styles->label(),
        ]));
    }
    $form_state->setRedirectUrl($styles->toUrl('collection'));
  }

}
